/*
 * Sonar C++ Plugin (Community)
 * Copyright (C) 2010 Neticoa SAS France
 * dev@sonar.codehaus.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.cxx.utils;

import org.sonar.api.batch.Sensor;
import org.sonar.api.batch.SensorContext;
import org.sonar.api.resources.InputFile;
import org.sonar.api.resources.Project;
import org.sonar.plugins.cxx.CxxLanguage;

import java.io.File;
import java.util.List;

/**
 * Common sensor class that works with files
 * @author Przemyslaw Kociolek
 */
public abstract class CxxFileSensor implements Sensor {

  private CxxLanguage language = new CxxLanguage();

  public boolean shouldExecuteOnProject(Project project) {
    return CxxLanguage.KEY.equals(project.getLanguageKey());
  }

  public void analyse(Project project, SensorContext context) {
    List<InputFile> sourceFiles = project.getFileSystem().mainFiles(CxxLanguage.KEY);

    for (InputFile inputFile : sourceFiles) {
      if (shouldParseFile(inputFile.getFile())) {
        parseFile(inputFile, project, context);
      }
    }
  }

  protected boolean shouldParseFile(File file) {
    for (String suffix : language.getSourceFileSuffixes()) {
      if (file.getAbsolutePath().endsWith(suffix)) {
        return true;
      }
    }
    return false;
  }

  protected abstract void parseFile(InputFile file, Project project, SensorContext context);

}
